> {-# OPTIONS -XMultiParamTypeClasses
> #-}

> module Tree
> where
> import Stdenv hiding (even, odd)
> import qualified Prelude as Stdenv
> import Ratio
> import Memo
> import Idiom
> import Stream hiding (main, power, bin, tree)
> import NumExt

%------------------------------------------------------------------------------
\section{Infinite trees}
%------------------------------------------------------------------------------

> data Tree α = Node { root :: α, left :: Tree α, right :: Tree α }

---

Inifinite trees of type |Tree a| are memoised functions of type |Binary -> α|.

< Binary -> α  ≅  Tree α

> data Binary  =  Nil | One Binary | Two Binary

> instance Memo Binary Tree where
>   tabulate f  =  Node (f Nil) (tabulate (f . One)) (tabulate (f . Two))
>
>   lookup t Nil      =  root t
>   lookup t (One n)  =  lookup (left  t) n
>   lookup t (Two n)  =  lookup (right t) n

---

Infinite trees are idioms aka applicative functors.

> instance Idiom Tree where
>   pure a  =  t where t = Node a t t
>   t ◇ u   =  Node ((root t) (root u)) (left t ◇ left u) (right t ◇ right u)
>   map f t =  Node (f (root t)) (map f (left t)) (map f (right t))

---

Instance declarations.

Showing infinite trees and testing for equality.

> depth  ::  Int
> depth  =   5

> showTree d t = putStr (showsTree d t "")

> showsTree d = showt d
>   where
>   showt 0       t  =  showString ""
>   showt (n + 1) t  =  showt n (left t) .
>                       showString (replicate (4 * (d - n - 1)) ' ') . shows (root t) . showString "\n" .
>                       showt n (right t)

> instance (Show a) => Show (Tree a) where
>    showsPrec _ t = showsTree depth t

> equal :: (Eq α) => Int -> Tree α -> Tree α -> Bool
> equal 0       t u  =  root t == root u
> equal (n + 1) t u  =  root t == root u && equal n (left  t) (left  u) 
>                                        && equal n (right t) (right u)

> instance (Eq a) => Eq (Tree a) where
>   t == u  =  equal depth t u  -- HACK
> instance (Ord a) => Ord (Tree a) where
>   t <= u  =  error "TODO"

`Generic' instances for |Functor| and numeric classes.

> instance Functor Tree where
>   fmap  =  map
> instance (Num a) => Num (Tree a) where
>   (+)          =  zip (+)
>   (-)          =  zip (-)
>   (*)          =  zip (*)
>   negate       =  map negate
>   abs          =  map abs
>   signum       =  map signum
>   fromInteger  =  repeat . fromInteger
> instance (Enum a) => Enum (Tree a) where
>   toEnum i    =  repeat (toEnum i)
>   fromEnum s  =  fromEnum (root s)  -- HACK?!
> instance (Real a) => Real (Tree a) where
>   toRational s  =  toRational (root s)  -- HACK?!
> instance (Integral a) => Integral (Tree a) where
>   div          =  zip div
>   mod          =  zip mod
>   quotRem s t  =  unzip (zip quotRem s t)
>   toInteger s  =  toInteger (root s)  -- HACK?!
> instance (Fractional a) => Fractional (Tree a) where
>   s / t           =  zip (Stdenv./) s t
>   recip s         =  map recip s
>   fromRational r  =  repeat (fromRational r)

> instance (NumExt a) => NumExt (Tree a) where
>   (^)     =  zip (^)
>   (/)     =  zip (/)
>   fact    =  map fact
>   fall    =  zip fall
>   choose  =  zip choose

> infixl 7 %^
> t %^ u = zip (%) t u

---

Recursion and iteration.

> recurse :: (α -> α) -> (α -> α) -> (α -> Tree α)
> recurse f g a = t
>   where t = Node a (map f t) (map g t)

> branch :: (α -> α) -> (α -> α) -> (α -> Tree α)
> branch f g a = loop a
>   where loop x = Node x (loop (f x)) (loop (g x))

%------------------------------------------------------------------------------
\section{Examples}
%------------------------------------------------------------------------------

> bin  =  Node 0 (2 * bin + 1) (2 * bin + 2)

> pos  =  Node 1 (2 * pos + 0) (2 * pos + 1)

> stern  ::  Tree Rational
> stern  =   Node 1 (1 / (1 / stern + 1)) (stern + 1)

> lbits  =  Node []  (map ([0] ++) lbits) (map ([1] ++) lbits)
>
> rbits  =  Node []  (map (++ [0]) rbits) (map (++ [1]) rbits)

> fusc   =  Node 1 fusc (fusc + fusc')
> fusc'  =  Node 1 (fusc + fusc') fusc'

%------------------------------------------------------------------------------
\section{Conversion to and from streams}
%------------------------------------------------------------------------------

> stream    ::  Tree α -> Stream α
> stream t  =   root t ≺ stream (chop t)
 
> chop    ::  Tree α -> Tree α
> chop t  =   Node (root (left t)) (right t) (chop (left t))

Converting between trees and streams: |head| is |root|, |tail| is
|chop| and |≺| is |cons|.

> cons      ::  α -> Tree α -> Tree α
> cons a t  =   Node a (cons (root t) (right t)) (left t)

---

> tree    ::  Stream α -> Tree α
> tree s  =   Node (head s) (tree (even (tail s))) (tree (odd (tail s)))

%------------------------------------------------------------------------------
\section{Monoids}
%------------------------------------------------------------------------------

> class Monoid α where
>   mempty   ::  α
>   mappend  ::  α -> α -> α

> ε  ::  (Monoid α) => α
> ε  =   mempty
> infixr 5 ·
> (·)  ::  (Monoid α) => α -> α -> α
> (·)  =   mappend

%------------------------------------------------------------------------------
\section{Vectors and matrices}
%------------------------------------------------------------------------------

> data Matrix = Mat Integer Integer Integer Integer
>   deriving (Show, Eq)

> data Vector = Vec Integer Integer
>   deriving (Show, Eq)

> instance Num Matrix where
>   fromInteger n = Mat n 0 0 n
>
>   Mat a b c d + Mat a' b' c' d'
>     =  Mat (a+a') (b+b') (c+c') (d+d')
>
>   Mat a b c d - Mat a' b' c' d'
>     =  Mat (a-a') (b-b') (c-c') (d-d')
>
>   Mat a b c d * Mat a' b' c' d'
>     =  Mat (a*a' + b*c') (a*b' + b*d') (c*a' + d*c') (c*b' + d*d')
>
>   negate (Mat a b c d)
>     =  Mat (negate a) (negate b) (negate c) (negate d)
>
>   abs (Mat a b c d)
>     =  Mat (abs a) (abs b) (abs c) (abs d)
>
>   signum (Mat a b c d)
>     =  Mat (signum a) (signum b) (signum c) (signum d)

> transpose (Mat a b c d) = Mat a c b d

> det (Mat a b c d) = a*d - b*c

Matrix-vector multiplication.

> Mat a b c d *. Vec x y = Vec (a * x + b * y) (c * x + d * y)
> (x, y) .* Mat a b c d = (a * x + c * y, b * x + d * y)

> instance Monoid Matrix where
>   mempty   =  1
>   mappend  =  (*)

